import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'file_picker_platform_interface.dart';

const MethodChannel _channel = MethodChannel('miguelruivo.flutter.plugins.filepicker');

/// An implementation of [FilePickerPlatform] that uses method channels.
class MethodChannelFilePicker extends FilePickerPlatform {
  static const String _tag = 'MethodChannelFilePicker';

  Future<Map<String, String>> getMultiFilePath({
    FileType type = FileType.any,
    List<String> allowedExtensions,
  }) async =>
      await _getPath(describeEnum(type), true, allowedExtensions);

  Future<String> getFilePath({
    FileType type = FileType.any,
    List<String> allowedExtensions,
  }) async =>
      await _getPath(describeEnum(type), false, allowedExtensions);

  Future<dynamic> getFile({
    FileType type = FileType.any,
    List<String> allowedExtensions,
  }) async =>
      await _getPath(describeEnum(type), false, allowedExtensions);

  Future<List<dynamic>> getMultiFile({
    FileType type = FileType.any,
    List<String> allowedExtensions,
  }) async =>
      await _getPath(describeEnum(type), true, allowedExtensions);

  Future<bool> clearTemporaryFiles() async => _channel.invokeMethod<bool>('clear');

  Future<dynamic> _getPath(
    String type,
    bool allowMultipleSelection,
    List<String> allowedExtensions,
  ) async {
    if (type != 'custom' && (allowedExtensions?.isNotEmpty ?? false)) {
      throw Exception('If you are using a custom extension filter, please use the FileType.custom instead.');
    }
    try {
      dynamic result = await _channel.invokeMethod(type, {
        'allowMultipleSelection': allowMultipleSelection,
        'allowedExtensions': allowedExtensions,
      });
      if (result != null && allowMultipleSelection) {
        if (result is String) {
          result = [result];
        }
        return Map<String, String>.fromIterable(result, key: (path) => path.split('/').last, value: (path) => path);
      }
      return result;
    } on PlatformException catch (e) {
      print('[$_tag] Platform exception: $e');
      rethrow;
    } catch (e) {
      print('[$_tag] Unsupported operation. Method not found. The exception thrown was: $e');
      rethrow;
    }
  }
}
